#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

// NeoPixel Setup
int neoPixelPin = 6;
int numPixels = 64;
int brightness = 5; 
Adafruit_NeoPixel strip = Adafruit_NeoPixel(numPixels, neoPixelPin, NEO_GRB + NEO_KHZ800);

// Variables for NeoPixel
int r = 0, g = 0, b = 0;
int dt = 500;  // Delay time between images
int r0, r1, r2, r3, r4, r5, r6, r7;

// DFPlayer Setup
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
SoftwareSerial mySoftwareSerial(10, 11); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

// Variables for MP3 playback
unsigned long trackTimer = millis();
bool firstPlayDone = false;  // Flag to track if track 7 has been played
int currentTrack = 7;  // Start with track 7
unsigned long trackStartTime = 0;

void setup() {
  // DFPlayer initialization
  mySoftwareSerial.begin(9600);
  Serial.begin(115200);

  Serial.println();
  Serial.println(F("DFRobot DFPlayer Mini Demo"));
  Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));

  if (!myDFPlayer.begin(mySoftwareSerial)) {  // Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1. Please recheck the connection!"));
    Serial.println(F("2. Please insert the SD card!"));
    while(true);  // Stay in an infinite loop if initialization fails.
  }
  Serial.println(F("DFPlayer Mini online."));
  
  myDFPlayer.volume(15);  // Set volume value (0-30).
  myDFPlayer.play(currentTrack);  // Start by playing track 7

  // Store the start time of the track
  trackStartTime = millis();

  // NeoPixel initialization
  strip.begin();         // Initialize the NeoPixel strip
  strip.setBrightness(brightness);  // Set the brightness
  strip.show();          // Ensure the NeoPixels are off at start
}

void loop() {
  // Image display logic for NeoPixel Matrix
  static unsigned long imageTimer = millis();
  
  if (millis() - imageTimer > dt) {
    imageTimer = millis();  // Reset the timer

    // IMAGE 1
    clear_matrix(); 
    r = 255;
    r0 = 0; r1 = 66; r2 = 102; r3 = 24; r4 = 24; r5 = 102; r6 = 66; r7 = 0; //červené X
    image();

    r = 255; g = 255; r0=48;r2=8;r3=160;r4=132;r5=144;r6=0;r7=28;
    image();
    delay(dt);

    // IMAGE 2 (Repeat this structure for other images as needed)
    clear_matrix(); 
    r = 255;
    r0 = 0; r1 = 66; r2 = 102; r3 = 24; r4 = 24; r5 = 102; r6 = 66; r7 = 0; //červené X
    image();
    r = 255; g = 255; r0=24;r1=0;r2=144;r3=132;r4=33;r5=8;r6=0;r7=112;
    image();
    delay(dt);

    // IMAGE3

clear_matrix(); r=255;
r0 = 0; r1 = 66; r2 = 102; r3 = 24; r4 = 24; r5 = 102; r6 = 66; r7 = 0; //červené X
image();

r=255;g=255;
r0=6;r1=144;r2=0;r3=2;r4=64;r5=1;r6=8;r7=224;

image();delay(dt);

// IMAGE4

clear_matrix(); r=255;
r0 = 0; r1 = 66; r2 = 102; r3 = 24; r4 = 24; r5 = 102; r6 = 66; r7 = 0; //červené X
image();

//Set4
r=255;g=255;
r0=195;r1=8;r2=0;r3=64;r4=2;r5=0;r6=145;r7=193;

image();delay(dt);

// IMAGE5

clear_matrix(); r=255;
r0 = 0; r1 = 66; r2 = 102; r3 = 24; r4 = 24; r5 = 102; r6 = 66; r7 = 0; //červené X
image();

//Set5
r=255;g=255;
r0=112;r1=1;r2=9;r3=160;r4=132;r5=144;r6=0;r7=6;

image();delay(dt);

// IMAGE6

clear_matrix(); r=255;
r0 = 0; r1 = 66; r2 = 102; r3 = 24; r4 = 24; r5 = 102; r6 = 66; r7 = 0; //červené X
image();

//Set6
r=255;g=255;
r0=48;r1=0;r2=144;r3=5;r4=33;r5=9;r6=0;r7=14;

image();delay(dt);

  }

  // MP3 Playback Logic: Play a random track when the current track finishes
  if (millis() - trackStartTime > myDFPlayer.readState() && myDFPlayer.readState() == 0) {  // Check if the track has finished
    // If track 7 has been played and track 1 is not yet played, play track 1
    if (!firstPlayDone) {
      myDFPlayer.play(1);  // Play track 1 after track 7
      currentTrack = 1;
      firstPlayDone = true;
    } else {
      // Play a random track between 2 and 6
      currentTrack = random(2, 7);  // random(2, 7) gives a number between 2 and 6
      myDFPlayer.play(currentTrack);  // Play the randomly selected track
    }

    // Update the track start time for the next track
    trackStartTime = millis();
  }
}

//*****************Subs for NeoPixel Images******************

void image() {
  for (int i = 0; i < 8; i++) {
    if (bitRead(r0, i) == 1) strip.setPixelColor(i, r, g, b);
    if (bitRead(r1, i) == 1) strip.setPixelColor(i + 8, r, g, b);
    if (bitRead(r2, i) == 1) strip.setPixelColor(i + 16, r, g, b);
    if (bitRead(r3, i) == 1) strip.setPixelColor(i + 24, r, g, b);
    if (bitRead(r4, i) == 1) strip.setPixelColor(i + 32, r, g, b);
    if (bitRead(r5, i) == 1) strip.setPixelColor(i + 40, r, g, b);
    if (bitRead(r6, i) == 1) strip.setPixelColor(i + 48, r, g, b);
    if (bitRead(r7, i) == 1) strip.setPixelColor(i + 56, r, g, b);
  }
  strip.show();
  r0 = 0; r1 = 0; r2 = 0; r3 = 0; r4 = 0; r5 = 0; r6 = 0; r7 = 0;
  r = 0; g = 0; b = 0;
}

void clear_matrix() {
  r = 0; g = 0; b = 0;
  strip.clear();  // Initialize all pixels to 'off'
  strip.show();   // Make sure it is visible
}
